﻿using System;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Drawing;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WmsServerEdition;

namespace WorldMapKitPlugin
{
    public class WorldMapKitPlugin : WmsLayerPlugin
    {
        public WorldMapKitPlugin()
            : base()
        {
            BrowerCacheExpiration = TimeSpan.FromDays(30);
        }

        // This method is only called once per style and crs.  In it you should create your
        // layers and add them to the MapConfiguration.  If you want to use tile caching you
        // can also specify that in the MapConfiguration under the TileCache property.
        // If you have setup multiple styles or projections this method will get called for
        // each unique combination
        protected override MapConfiguration GetMapConfigurationCore(string style, string crs)
        {
            // Create the World Map Kit render layer from a specified worldmapkit data folder
            WorldMapKitRenderLayer worldMapKitRenderLayer = null;

            // Create a TileCache object and add it to mapConfiguration
            FileBitmapTileCache tileCache = new FileBitmapTileCache(ConfigurationManager.AppSettings["WorldMapKitCache"], "WorldMapKit", TileImageFormat.Jpeg, new MapSuiteTileMatrix(590591790));

            // If the SRS of request is for Spherical Mercator projection and so we need to project the data
            switch (crs)
            {
                case "EPSG:900913":
                    worldMapKitRenderLayer = new WorldMapKitRenderLayer(ConfigurationManager.AppSettings["WorldMapKitSphericalMercatorData"]);
                    tileCache.CacheId = "WorldMapKitSphericalMercator";
                    break;
                case "EPSG:4326":
                case "CRS:84":
                    worldMapKitRenderLayer = new WorldMapKitRenderLayer(ConfigurationManager.AppSettings["WorldMapKitData"]);
                    tileCache.CacheId = "WorldMapKitDecimalDegrees";
                    break;
                default:
                    throw new ArgumentException("The input CRS is not supported by now, we only support EPSG:4326 and EPSG:900913 and CRS:84.");
            }

            // Create a MapConfiguration and add layer to it.
            MapConfiguration mapConfiguration = new MapConfiguration();
            mapConfiguration.Layers.Add(worldMapKitRenderLayer);
            mapConfiguration.TileCache = tileCache;

            return mapConfiguration;
        }

        protected override GeographyUnit GetGeographyUnitCore(string crs)
        {
            GeographyUnit geographyUnit = GeographyUnit.Unknown;
            switch (crs)
            {
                case "EPSG:4326":
                case "CRS:84":
                    geographyUnit = GeographyUnit.DecimalDegree;
                    break;
                case "EPSG:900913":
                    geographyUnit = GeographyUnit.Meter;
                    break;
                default:
                    throw new ArgumentException("The input CRS is not supported by now, we only support EPSG:4326 and EPSG:900913 and CRS:84.");
            }

            return geographyUnit;
        }

        // In this method you need to return the name of the Layer that WMS will expose.
        // You will use this name on the client to specify the layer you want to consume
        protected override string GetNameCore()
        {
            return "WorldMapKitLayer";
        }

        // In this method you need to return the projections that are supported by your data.
        // It is your responsability to project the data in the MapConfiguration for each projection
        // type you specify here.
        protected override Collection<string> GetProjectionsCore()
        {
            return new Collection<string> { "EPSG:4326", "EPSG:900913", "CRS:84" };
        }

        // In this method you need to return the styels that are supported by your data.
        protected override Collection<WmsLayerStyle> GetStylesCore()
        {
            WmsLayerStyle style = new WmsLayerStyle("WorldMapKitDefaultStyle");
            WmsLayerStyle style1 = new WmsLayerStyle("WorldMapKitDefaultStyles");
            return new Collection<WmsLayerStyle> { style, style1 };
        }

        // In this method you need to return the bounding box of the layer.
        protected override RectangleShape GetBoundingBoxCore(string crs)
        {
            RectangleShape boundingBox = null;
            switch (crs)
            {
                case "EPSG:4326":
                case "CRS:84":
                    boundingBox = new RectangleShape(-180, 90, 180, -90);
                    break;
                case "EPSG:900913":
                    boundingBox = new RectangleShape(-20037508.34, 20037508.34, 20037508.34, -20037508.34);
                    break;
                default:
                    throw new ArgumentException("The input CRS is not supported by now, we only support EPSG:4326 and EPSG:900913 and CRS:84.");
            }

            return boundingBox;
        }

        protected override System.Drawing.Bitmap GetMapCore(GetMapRequest getMapRequest, MapConfiguration mapConfiguration, System.Web.HttpContext context)
        {
            string imageFormat = context.Request.QueryString["Format"];
            string userName = context.Request.QueryString["UserName"];

            //BackgroundLayer backgroundLayer = mapConfiguration.Layers["Background"] as BackgroundLayer;
            //if (backgroundLayer != null) backgroundLayer.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 153, 179, 204));

            Bitmap mapImage = null;
            bool userValidated = false;//SecurityHelper.AuthenticateUserRequest(context);
            ShapeFileFeatureLayer layer = mapConfiguration.Layers[0] as ShapeFileFeatureLayer;

            //---------------------------------------------------------------------------
            var baseDir = "e:\\Sample Templates";

            var TilesPath = baseDir + "Tiles";
            var savedTilePath = baseDir + "\\SavedTile.png";
            var fileCache = new FileBitmapTileCache(TilesPath);
            fileCache.TileAccessMode = TileAccessMode.ReadOnly;
            fileCache.CacheId = "tilecache";
            mapConfiguration.TileCache = fileCache;
            //---------------------------------------------------------------------------

            if (layer != null)
            {
                layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1; //GetAreaStyleByUserName(userName, userValidated);
                mapImage = base.GetMapCore(getMapRequest, mapConfiguration, context);
            }
            if (!userValidated)
            {
                mapImage = mapImage ?? base.GetMapCore(getMapRequest, mapConfiguration, context);
                PointShape centerPoint = getMapRequest.BoundingBox.GetCenterPoint();
                GdiPlusGeoCanvas canvas = new GdiPlusGeoCanvas();
                canvas.BeginDrawing(mapImage, getMapRequest.BoundingBox, GetGeographyUnit(getMapRequest.Crs));
                canvas.DrawTextWithWorldCoordinate("Your watermark here", new GeoFont("Arial", 11, DrawingFontStyles.Bold), new GeoSolidBrush(new GeoColor(100, GeoColor.StandardColors.Purple)), centerPoint.X, centerPoint.Y, DrawingLevel.LevelOne);
                canvas.EndDrawing();
            }
            //---------------------------------------------------------------------------
            mapImage.Save(savedTilePath);
            //---------------------------------------------------------------------------
            return mapImage;
        }
    }
}
